@@ -14,22 +14,23 @@
 
 """Base backend for traces
 
-See the docstring for pymc3.backends for more information (including
-creating custom backends).
+See the docstring for pymc3.backends for more information
 """
 import itertools as itl
 import logging
-from typing import Dict, List, Optional
+import warnings
+
 from abc import ABC
+from typing import List
 
 import numpy as np
-import warnings
 import theano.tensor as tt
 
-from ..model import modelcontext, Model
+from ..model import modelcontext
+from ..util import get_var_name
 from .report import SamplerReport, merge_reports
 
-logger = logging.getLogger('pymc3')
+logger = logging.getLogger("pymc3")
 
 
 class BackendError(Exception):
@@ -74,10 +75,8 @@ class BaseTrace(ABC):
             test_point_.update(test_point)
             test_point = test_point_
         var_values = list(zip(self.varnames, self.fn(test_point)))
-        self.var_shapes = {var: value.shape
-                           for var, value in var_values}
-        self.var_dtypes = {var: value.dtype
-                           for var, value in var_values}
+        self.var_shapes = {var: value.shape for var, value in var_values}
+        self.var_dtypes = {var: value.dtype for var, value in var_values}
         self.chain = None
         self._is_base_setup = False
         self.sampler_vars = None
@@ -103,13 +102,12 @@ class BaseTrace(ABC):
         for stats in sampler_vars:
             for key, dtype in stats.items():
                 if dtypes.setdefault(key, dtype) != dtype:
-                    raise ValueError("Sampler statistic %s appears with "
-                                     "different types." % key)
+                    raise ValueError("Sampler statistic %s appears with " "different types." % key)
 
         self.sampler_vars = sampler_vars
 
     # pylint: disable=unused-argument
-    def setup(self, draws, chain, sampler_vars=None) -> None: 
+    def setup(self, draws, chain, sampler_vars=None) -> None:
         """Perform chain-specific setup.
 
         Parameters
@@ -154,7 +152,7 @@ class BaseTrace(ABC):
         try:
             return self.point(int(idx))
         except (ValueError, TypeError):  # Passed variable or variable name.
-            raise ValueError('Can only index with slice or integer')
+            raise ValueError("Can only index with slice or integer")
 
     def __len__(self):
         raise NotImplementedError
@@ -198,13 +196,13 @@ class BaseTrace(ABC):
         if sampler_idx is not None:
             return self._get_sampler_stats(stat_name, sampler_idx, burn, thin)
 
-        sampler_idxs = [i for i, s in enumerate(self.sampler_vars)
-                        if stat_name in s]
+        sampler_idxs = [i for i, s in enumerate(self.sampler_vars) if stat_name in s]
         if not sampler_idxs:
             raise KeyError("Unknown sampler stat %s" % stat_name)
 
-        vals = np.stack([self._get_sampler_stats(stat_name, i, burn, thin)
-                         for i in sampler_idxs], axis=-1)
+        vals = np.stack(
+            [self._get_sampler_stats(stat_name, i, burn, thin) for i in sampler_idxs], axis=-1
+        )
         if vals.shape[-1] == 1:
             return vals[..., 0]
         else:
@@ -276,14 +274,14 @@ class MultiTrace:
 
     Attributes
     ----------
-        nchains: int
-            Number of chains in the `MultiTrace`.
-        chains: `List[int]`
-            List of chain indices
-        report: str
-            Report on the sampling process.
-        varnames: `List[str]`
-            List of variable names in the trace(s)
+    nchains: int
+        Number of chains in the `MultiTrace`.
+    chains: `List[int]`
+        List of chain indices
+    report: str
+        Report on the sampling process.
+    varnames: `List[str]`
+        List of variable names in the trace(s)
     """
 
     def __init__(self, straces):
@@ -295,13 +293,12 @@ class MultiTrace:
 
         self._report = SamplerReport()
         for strace in straces:
-            if hasattr(strace, '_warnings'):
+            if hasattr(strace, "_warnings"):
                 self._report._add_warnings(strace._warnings, strace.chain)
 
     def __repr__(self):
-        template = '<{}: {} chains, {} iterations, {} variables>'
-        return template.format(self.__class__.__name__,
-                               self.nchains, len(self), len(self.varnames))
+        template = "<{}: {} chains, {} iterations, {} variables>"
+        return template.format(self.__class__.__name__, self.nchains, len(self), len(self.varnames))
 
     @property
     def nchains(self):
@@ -335,19 +332,20 @@ class MultiTrace:
             var = idx
             burn, thin = 0, 1
 
-        var = str(var)
+        var = get_var_name(var)
         if var in self.varnames:
             if var in self.stat_names:
-                warnings.warn("Attribute access on a trace object is ambigous. "
-                              "Sampler statistic and model variable share a name. Use "
-                              "trace.get_values or trace.get_sampler_stats.")
+                warnings.warn(
+                    "Attribute access on a trace object is ambigous. "
+                    "Sampler statistic and model variable share a name. Use "
+                    "trace.get_values or trace.get_sampler_stats."
+                )
             return self.get_values(var, burn=burn, thin=thin)
         if var in self.stat_names:
             return self.get_sampler_stats(var, burn=burn, thin=thin)
         raise KeyError("Unknown variable %s" % var)
 
-    _attrs = set(['_straces', 'varnames', 'chains', 'stat_names',
-                  'supports_sampler_stats', '_report'])
+    _attrs = {"_straces", "varnames", "chains", "stat_names", "supports_sampler_stats", "_report"}
 
     def __getattr__(self, name):
         # Avoid infinite recursion when called before __init__
@@ -355,17 +353,18 @@ class MultiTrace:
         if name in self._attrs:
             raise AttributeError
 
-        name = str(name)
+        name = get_var_name(name)
         if name in self.varnames:
             if name in self.stat_names:
-                warnings.warn("Attribute access on a trace object is ambigous. "
-                              "Sampler statistic and model variable share a name. Use "
-                              "trace.get_values or trace.get_sampler_stats.")
+                warnings.warn(
+                    "Attribute access on a trace object is ambigous. "
+                    "Sampler statistic and model variable share a name. Use "
+                    "trace.get_values or trace.get_sampler_stats."
+                )
             return self.get_values(name)
         if name in self.stat_names:
             return self.get_sampler_stats(name)
-        raise AttributeError("'{}' object has no attribute '{}'".format(
-            type(self).__name__, name))
+        raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
 
     def __len__(self):
         chain = self.chains[-1]
@@ -416,7 +415,7 @@ class MultiTrace:
                     self.varnames.remove(k)
                     new_var = 0
                 else:
-                    raise ValueError("Variable name {} already exists.".format(k))
+                    raise ValueError(f"Variable name {k} already exists.")
 
             self.varnames.append(k)
 
@@ -424,10 +423,12 @@ class MultiTrace:
             l_samples = len(self) * len(self.chains)
             l_v = len(v)
             if l_v != l_samples:
-                warnings.warn("The length of the values you are trying to "
-                              "add ({}) does not match the number ({}) of "
-                              "total samples in the trace "
-                              "(chains * iterations)".format(l_v, l_samples))
+                warnings.warn(
+                    "The length of the values you are trying to "
+                    "add ({}) does not match the number ({}) of "
+                    "total samples in the trace "
+                    "(chains * iterations)".format(l_v, l_samples)
+                )
 
             v = np.squeeze(v.reshape(len(chains), len(self), -1))
 
@@ -447,7 +448,7 @@ class MultiTrace:
         """
         varnames = self.varnames
         if name not in varnames:
-            raise KeyError("Unknown variable {}".format(name))
+            raise KeyError(f"Unknown variable {name}")
         self.varnames.remove(name)
         chains = self._straces
         for chain in chains.values():
@@ -456,8 +457,7 @@ class MultiTrace:
                     chain.vars.remove(va)
                     del chain.samples[name]
 
-    def get_values(self, varname, burn=0, thin=1, combine=True, chains=None,
-                   squeeze=True):
+    def get_values(self, varname, burn=0, thin=1, combine=True, chains=None, squeeze=True):
         """Get values from traces.
 
         Parameters
@@ -482,16 +482,14 @@ class MultiTrace:
         """
         if chains is None:
             chains = self.chains
-        varname = str(varname)
+        varname = get_var_name(varname)
         try:
-            results = [self._straces[chain].get_values(varname, burn, thin)
-                       for chain in chains]
+            results = [self._straces[chain].get_values(varname, burn, thin) for chain in chains]
         except TypeError:  # Single chain passed.
             results = [self._straces[chains].get_values(varname, burn, thin)]
         return _squeeze_cat(results, combine, squeeze)
 
-    def get_sampler_stats(self, stat_name, burn=0, thin=1, combine=True,
-                          chains=None, squeeze=True):
+    def get_sampler_stats(self, stat_name, burn=0, thin=1, combine=True, chains=None, squeeze=True):
         """Get sampler statistics from the trace.
 
         Parameters
@@ -519,8 +517,9 @@ class MultiTrace:
         except TypeError:
             chains = [chains]
 
-        results = [self._straces[chain].get_sampler_stats(stat_name, None, burn, thin)
-                   for chain in chains]
+        results = [
+            self._straces[chain].get_sampler_stats(stat_name, None, burn, thin) for chain in chains
+        ]
         return _squeeze_cat(results, combine, squeeze)
 
     def _slice(self, slice):
@@ -581,7 +580,9 @@ def merge_traces(mtraces: List[MultiTrace]) -> MultiTrace:
     base_mtrace = mtraces[0]
     chain_len = len(base_mtrace)
     # check base trace
-    if any(len(st) != chain_len for _, st in base_mtrace._straces.items()):  # pylint: disable=line-too-long
+    if any(
+        len(st) != chain_len for _, st in base_mtrace._straces.items()
+    ):  # pylint: disable=line-too-long
         raise ValueError("Chains are of different lengths.")
     for new_mtrace in mtraces[1:]:
         for new_chain, strace in new_mtrace._straces.items():
