@@ -1,48 +1,50 @@
+import contextvars
+import logging
 import numbers
+import warnings
+
+from collections import UserDict
+from contextlib import AbstractContextManager
 from typing import (
-    List,
-    Dict,
+    TYPE_CHECKING,
     Any,
+    Callable,
+    Dict,
+    List,
     Optional,
+    Set,
     Tuple,
     Union,
     cast,
-    TYPE_CHECKING,
-    Callable,
     overload,
-    Set,
 )
-import warnings
-import logging
-from collections import UserDict
-from contextlib import AbstractContextManager
-import contextvars
-from typing_extensions import Protocol, Literal
 
 import numpy as np
 import theano
 import theano.tensor as tt
-from xarray import Dataset
+
 from arviz import InferenceData
+from typing_extensions import Literal, Protocol
+from xarray import Dataset
 
 from ..backends.base import MultiTrace
+from ..exceptions import IncorrectArgumentsError
+from ..model import (
+    Model,
+    MultiObservedRV,
+    ObservedRV,
+    get_named_nodes_and_relations,
+    modelcontext,
+)
+from ..util import chains_and_samples, dataset_to_point_list, get_var_name
+from ..vartypes import theano_constant
 from .distribution import (
+    _compile_theano_function,
     _DrawValuesContext,
     _DrawValuesContextBlocker,
     is_fast_drawable,
-    _compile_theano_function,
     vectorized_ppc,
 )
-from ..model import (
-    Model,
-    get_named_nodes_and_relations,
-    ObservedRV,
-    MultiObservedRV,
-    modelcontext,
-)
-from ..exceptions import IncorrectArgumentsError
-from ..vartypes import theano_constant
-from ..util import dataset_to_point_dict, chains_and_samples
 
 # Failing tests:
 #    test_mixture_random_shape::test_mixture_random_shape
@@ -81,15 +83,11 @@ class _TraceDict(_TraceDictParent):
         multi_trace: Optional[MultiTrace] = None,
         dict: Optional[Dict[str, np.ndarray]] = None,
     ):
-        """
-
-        """
+        """"""
         if multi_trace:
             assert point_list is None and dict is None
             self.data = {}  # Dict[str, np.ndarray]
-            self._len = sum(
-                (len(multi_trace._straces[chain]) for chain in multi_trace.chains)
-            )
+            self._len = sum(len(multi_trace._straces[chain]) for chain in multi_trace.chains)
             self.varnames = multi_trace.varnames
             for vn in multi_trace.varnames:
                 self.data[vn] = multi_trace.get_values(vn)
@@ -121,11 +119,7 @@ class _TraceDict(_TraceDictParent):
             self.data = dict
             self.varnames = list(dict.keys())
             self._len = dict[self.varnames[0]].shape[0]
-        assert (
-            self.varnames is not None
-            and self._len is not None
-            and self.data is not None
-        )
+        assert self.varnames is not None and self._len is not None and self.data is not None
 
     def __len__(self) -> int:
         return self._len
@@ -153,15 +147,13 @@ class _TraceDict(_TraceDictParent):
 
     def __getitem__(self, item):
         if isinstance(item, str):
-            return super(_TraceDict, self).__getitem__(item)
+            return super().__getitem__(item)
         elif isinstance(item, slice):
             return self._extract_slice(item)
         elif isinstance(item, int):
-            return _TraceDict(
-                dict={k: np.atleast_1d(v[item]) for k, v in self.data.items()}
-            )
+            return _TraceDict(dict={k: np.atleast_1d(v[item]) for k, v in self.data.items()})
         elif hasattr(item, "name"):
-            return super(_TraceDict, self).__getitem__(item.name)
+            return super().__getitem__(item.name)
         else:
             raise IndexError("Illegal index %s for _TraceDict" % str(item))
 
@@ -219,10 +211,10 @@ def fast_sample_posterior_predictive(
 
     if isinstance(trace, InferenceData):
         nchains, ndraws = chains_and_samples(trace)
-        trace = dataset_to_point_dict(trace.posterior)
+        trace = dataset_to_point_list(trace.posterior)
     elif isinstance(trace, Dataset):
         nchains, ndraws = chains_and_samples(trace)
-        trace = dataset_to_point_dict(trace)
+        trace = dataset_to_point_list(trace)
     elif isinstance(trace, MultiTrace):
         nchains = trace.nchains
         ndraws = len(trace)
@@ -238,11 +230,9 @@ def fast_sample_posterior_predictive(
     with model:
 
         if keep_size and samples is not None:
-            raise IncorrectArgumentsError(
-                "Should not specify both keep_size and samples arguments"
-            )
+            raise IncorrectArgumentsError("Should not specify both keep_size and samples arguments")
 
-        if isinstance(trace, list) and all((isinstance(x, dict) for x in trace)):
+        if isinstance(trace, list) and all(isinstance(x, dict) for x in trace):
             _trace = _TraceDict(point_list=trace)
         elif isinstance(trace, MultiTrace):
             _trace = _TraceDict(multi_trace=trace)
@@ -306,20 +296,15 @@ def fast_sample_posterior_predictive(
         for s in _samples:
             strace = _trace if s == len_trace else _trace[slice(0, s)]
             try:
-                values = posterior_predictive_draw_values(
-                    cast(List[Any], vars), strace, s
-                )
-                new_trace: Dict[str, np.ndarray] = {
-                    k.name: v for (k, v) in zip(vars, values)
-                }
+                values = posterior_predictive_draw_values(cast(List[Any], vars), strace, s)
+                new_trace: Dict[str, np.ndarray] = {k.name: v for (k, v) in zip(vars, values)}
                 ppc_trace.extend_trace(new_trace)
             except KeyboardInterrupt:
                 pass
 
         if keep_size:
             return {
-                k: ary.reshape((nchains, ndraws, *ary.shape[1:]))
-                for k, ary in ppc_trace.items()
+                k: ary.reshape((nchains, ndraws, *ary.shape[1:])) for k, ary in ppc_trace.items()
             }
         # this gets us a Dict[str, np.ndarray] instead of my wrapped equiv.
         return ppc_trace.data
@@ -354,9 +339,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
     named_nodes_children: Dict[str, Any]
     _tok: contextvars.Token
 
-    def __init__(
-        self, vars, trace: _TraceDict, samples, model: Optional[Model], size=None
-    ):
+    def __init__(self, vars, trace: _TraceDict, samples, model: Optional[Model], size=None):
         if size is not None:
             raise NotImplementedError(
                 "sample_posterior_predictive does not support the size argument at this time."
@@ -454,13 +437,13 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
             # test_distributions_random::TestDrawValues::test_draw_order fails without it
             # The remaining params that must be drawn are all hashable
             to_eval: Set[int] = set()
-            missing_inputs: Set[int] = set([j for j, p in self.symbolic_params])
+            missing_inputs: Set[int] = {j for j, p in self.symbolic_params}
 
             while to_eval or missing_inputs:
                 if to_eval == missing_inputs:
                     raise ValueError(
                         "Cannot resolve inputs for {}".format(
-                            [str(trace.varnames[j]) for j in to_eval]
+                            [get_var_name(trace.varnames[j]) for j in to_eval]
                         )
                     )
                 to_eval = set(missing_inputs)
@@ -474,17 +457,12 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                         try:
                             if param in self.named_nodes_children:
                                 for node in self.named_nodes_children[param]:
-                                    if (
-                                        node.name not in givens
-                                        and (node, samples) in drawn
-                                    ):
+                                    if node.name not in givens and (node, samples) in drawn:
                                         givens[node.name] = (
                                             node,
                                             drawn[(node, samples)],
                                         )
-                            value = self.draw_value(
-                                param, trace=self.trace, givens=givens.values()
-                            )
+                            value = self.draw_value(param, trace=self.trace, givens=givens.values())
                             assert isinstance(value, np.ndarray)
                             self.evaluated[param_idx] = drawn[(param, samples)] = value
                             givens[param.name] = (param, value)
@@ -493,10 +471,10 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
         return [self.evaluated[j] for j in params]
 
     def init(self) -> None:
-        """This method carries out the initialization phase of sampling 
-    from the posterior predictive distribution.  Notably it initializes the
-    ``_DrawValuesContext`` bookkeeping object and evaluates the "fast drawable"
-    parts of the model."""
+        """This method carries out the initialization phase of sampling
+        from the posterior predictive distribution.  Notably it initializes the
+        ``_DrawValuesContext`` bookkeeping object and evaluates the "fast drawable"
+        parts of the model."""
         vars: List[Any] = self.vars
         trace: _TraceDict = self.trace
         samples: int = self.samples
@@ -519,11 +497,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                 if (var, samples) in drawn:
                     evaluated[i] = drawn[(var, samples)]
                     # We filter out Deterministics by checking for `model` attribute
-                elif (
-                    name is not None
-                    and hasattr(var, "model")
-                    and name in trace.varnames
-                ):
+                elif name is not None and hasattr(var, "model") and name in trace.varnames:
                     # param.name is in the trace.  Record it as drawn and evaluated
                     drawn[(var, samples)] = evaluated[i] = trace[cast(str, name)]
                 else:
@@ -567,7 +541,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
             The value or distribution. Constants or shared variables
             will be converted to an array and returned. Theano variables
             are evaluated. If `param` is a pymc3 random variable, draw
-            values from it and return that (as ``np.ndarray``), unless a 
+            values from it and return that (as ``np.ndarray``), unless a
             value is specified in the ``trace``.
         trace: pm.MultiTrace, optional
             A dictionary from pymc3 variable names to samples of their values
@@ -618,9 +592,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                 model = modelcontext(None)
                 assert isinstance(model, Model)
                 shape: Tuple[int, ...] = tuple(_param_shape(param, model))
-                return random_sample(
-                    param.random, param, point=trace, size=samples, shape=shape
-                )
+                return random_sample(param.random, param, point=trace, size=samples, shape=shape)
             elif (
                 hasattr(param, "distribution")
                 and hasattr(param.distribution, "random")
@@ -630,9 +602,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                     # shape inspection for ObservedRV
                     dist_tmp = param.distribution
                     try:
-                        distshape: Tuple[int, ...] = tuple(
-                            param.observations.shape.eval()
-                        )
+                        distshape: Tuple[int, ...] = tuple(param.observations.shape.eval())
                     except AttributeError:
                         distshape = tuple(param.observations.shape)
 
@@ -653,9 +623,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                         # we don't want to store these drawn values to the context
                         with _DrawValuesContextBlocker():
                             point = trace[0] if trace else None
-                            temp_val = np.atleast_1d(
-                                dist_tmp.random(point=point, size=None)
-                            )
+                            temp_val = np.atleast_1d(dist_tmp.random(point=point, size=None))
                         # if hasattr(param, 'name') and param.name == 'obs':
                         #     import pdb; pdb.set_trace()
                         # Sometimes point may change the size of val but not the
@@ -694,13 +662,9 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                     variables = values = []
                 # We only truly care if the ancestors of param that were given
                 # value have the matching dshape and val.shape
-                param_ancestors = set(
-                    theano.gof.graph.ancestors([param], blockers=list(variables))
-                )
+                param_ancestors = set(theano.gof.graph.ancestors([param], blockers=list(variables)))
                 inputs = [
-                    (var, val)
-                    for var, val in zip(variables, values)
-                    if var in param_ancestors
+                    (var, val) for var, val in zip(variables, values) if var in param_ancestors
                 ]
                 if inputs:
                     input_vars, input_vals = list(zip(*inputs))
@@ -709,9 +673,7 @@ class _PosteriorPredictiveSampler(AbstractContextManager):
                     input_vals = []
                 func = _compile_theano_function(param, input_vars)
                 if not input_vars:
-                    assert (
-                        input_vals == []
-                    )  # AFAICT if there are now vars, there can't be vals
+                    assert input_vals == []  # AFAICT if there are now vars, there can't be vals
                     output = func(*input_vals)
                     if hasattr(output, "shape"):
                         val = np.repeat(np.expand_dims(output, 0), samples, axis=0)
