@@ -13,38 +13,37 @@
 #   limitations under the License.
 
 import numpy as np
-import theano
-import pymc3 as pm
 import pytest
+import theano
+
+import pymc3 as pm
 
 
 @pytest.fixture(scope="function", autouse=True)
 def theano_config():
-    config = theano.configparser.change_flags(compute_test_value='raise')
+    config = theano.configparser.change_flags(compute_test_value="raise")
     with config:
         yield
 
 
-@pytest.fixture(scope='function', autouse=True)
+@pytest.fixture(scope="function", autouse=True)
 def exception_verbosity():
-    config = theano.configparser.change_flags(
-        exception_verbosity='high')
+    config = theano.configparser.change_flags(exception_verbosity="high")
     with config:
         yield
 
 
-@pytest.fixture(scope='function', autouse=False)
+@pytest.fixture(scope="function", autouse=False)
 def strict_float32():
-    if theano.config.floatX == 'float32':
-        config = theano.configparser.change_flags(
-            warn_float64='raise')
+    if theano.config.floatX == "float32":
+        config = theano.configparser.change_flags(warn_float64="raise")
         with config:
             yield
     else:
         yield
 
 
-@pytest.fixture(scope='function', autouse=False)
+@pytest.fixture(scope="function", autouse=False)
 def seeded_test():
     # TODO: use this instead of SeededTest
     np.random.seed(42)
