@@ -15,83 +15,87 @@
 import itertools
 import sys
 
-from .helpers import SeededTest, select_by_precision
-from ..vartypes import continuous_types
-from ..model import Model, Point, Deterministic
+import numpy as np
+import numpy.random as nr
+import pytest
+import scipy.stats
+import scipy.stats.distributions as sp
+import theano
+import theano.tensor as tt
+
+from numpy import array, exp, inf, log
+from numpy.testing import assert_allclose, assert_almost_equal, assert_equal
+from scipy import integrate
+from scipy.special import logit
+
+import pymc3 as pm
+
+from pymc3.theanof import floatX
+
 from ..blocking import DictToVarBijection
 from ..distributions import (
-    DensityDist,
-    Categorical,
-    Multinomial,
-    VonMises,
-    Dirichlet,
-    MvStudentT,
-    MvNormal,
-    MatrixNormal,
-    ZeroInflatedPoisson,
-    ZeroInflatedNegativeBinomial,
-    Constant,
-    Poisson,
+    AR1,
     Bernoulli,
     Beta,
     BetaBinomial,
-    HalfStudentT,
-    StudentT,
-    Weibull,
-    Pareto,
-    InverseGamma,
-    Gamma,
-    Cauchy,
-    HalfCauchy,
-    Lognormal,
-    Laplace,
-    NegativeBinomial,
-    Geometric,
-    Exponential,
-    ExGaussian,
-    Normal,
-    TruncatedNormal,
-    Flat,
-    LKJCorr,
-    Wald,
-    ChiSquared,
-    HalfNormal,
-    DiscreteUniform,
-    Bound,
-    Uniform,
-    Triangular,
     Binomial,
-    SkewNormal,
+    Bound,
+    Categorical,
+    Cauchy,
+    ChiSquared,
+    Constant,
+    DensityDist,
+    Dirichlet,
+    DiscreteUniform,
     DiscreteWeibull,
+    ExGaussian,
+    Exponential,
+    Flat,
+    Gamma,
+    Geometric,
     Gumbel,
-    Logistic,
-    OrderedLogistic,
-    LogitNormal,
-    Interpolated,
-    ZeroInflatedBinomial,
+    HalfCauchy,
     HalfFlat,
-    AR1,
+    HalfNormal,
+    HalfStudentT,
+    HyperGeometric,
+    Interpolated,
+    InverseGamma,
     KroneckerNormal,
-    Rice,
     Kumaraswamy,
+    Laplace,
+    LKJCorr,
+    Logistic,
+    LogitNormal,
+    Lognormal,
+    MatrixNormal,
     Moyal,
+    Multinomial,
+    MvNormal,
+    MvStudentT,
+    NegativeBinomial,
+    Normal,
+    OrderedLogistic,
+    Pareto,
+    Poisson,
+    Rice,
+    SkewNormal,
+    StudentT,
+    Triangular,
+    TruncatedNormal,
+    Uniform,
+    VonMises,
+    Wald,
+    Weibull,
+    ZeroInflatedBinomial,
+    ZeroInflatedNegativeBinomial,
+    ZeroInflatedPoisson,
+    continuous,
 )
-
-from ..distributions import continuous
-from pymc3.theanof import floatX
-from numpy import array, inf, log, exp
-from numpy.testing import assert_almost_equal, assert_allclose, assert_equal
-import numpy.random as nr
-import numpy as np
-import pytest
-
-from scipy import integrate
-import scipy.stats.distributions as sp
-import scipy.stats
-from scipy.special import logit
-import theano
-import theano.tensor as tt
 from ..math import kronecker
+from ..model import Deterministic, Model, Point
+from ..vartypes import continuous_types
+from .helpers import SeededTest, select_by_precision
 
 
 def get_lkj_cases():
@@ -156,9 +160,7 @@ class Domain:
             )
 
     def __neg__(self):
-        return Domain(
-            [-v for v in self.vals], self.dtype, (-self.lower, -self.upper), self.shape
-        )
+        return Domain([-v for v in self.vals], self.dtype, (-self.lower, -self.upper), self.shape)
 
 
 def product(domains, n_samples=-1):
@@ -175,10 +177,8 @@ def product(domains, n_samples=-1):
     try:
         names, domains = zip(*domains.items())
     except ValueError:  # domains.items() is empty
-        return []
-    all_vals = [
-        zip(names, val) for val in itertools.product(*[d.vals for d in domains])
-    ]
+        return [{}]
+    all_vals = [zip(names, val) for val in itertools.product(*[d.vals for d in domains])]
     if n_samples > 0 and len(all_vals) > n_samples:
         return (all_vals[j] for j in nr.choice(len(all_vals), n_samples, replace=False))
     return all_vals
@@ -427,9 +427,7 @@ def invlogit(x, eps=sys.float_info.epsilon):
 
 def orderedlogistic_logpdf(value, eta, cutpoints):
     c = np.concatenate(([-np.inf], cutpoints, [np.inf]))
-    ps = np.array(
-        [invlogit(eta - cc) - invlogit(eta - cc1) for cc, cc1 in zip(c[:-1], c[1:])]
-    )
+    ps = np.array([invlogit(eta - cc) - invlogit(eta - cc1) for cc, cc1 in zip(c[:-1], c[1:])])
     p = ps[value]
     return np.where(np.all(ps >= 0), np.log(p), -np.inf)
 
@@ -444,9 +442,7 @@ class Simplex:
 class MultiSimplex:
     def __init__(self, n_dependent, n_independent):
         self.vals = []
-        for simplex_value in itertools.product(
-            simplex_values(n_dependent), repeat=n_independent
-        ):
+        for simplex_value in itertools.product(simplex_values(n_dependent), repeat=n_independent):
             self.vals.append(np.vstack(simplex_value))
         self.shape = (n_independent, n_dependent)
         self.dtype = Unit.dtype
@@ -467,16 +463,12 @@ PdMatrix1 = Domain([np.eye(1), [[0.5]]], edges=(None, None))
 
 PdMatrix2 = Domain([np.eye(2), [[0.5, 0.05], [0.05, 4.5]]], edges=(None, None))
 
-PdMatrix3 = Domain(
-    [np.eye(3), [[0.5, 0.1, 0], [0.1, 1, 0], [0, 0, 2.5]]], edges=(None, None)
-)
+PdMatrix3 = Domain([np.eye(3), [[0.5, 0.1, 0], [0.1, 1, 0], [0, 0, 2.5]]], edges=(None, None))
 
 
 PdMatrixChol1 = Domain([np.eye(1), [[0.001]]], edges=(None, None))
 PdMatrixChol2 = Domain([np.eye(2), [[0.1, 0], [10, 1]]], edges=(None, None))
-PdMatrixChol3 = Domain(
-    [np.eye(3), [[0.1, 0, 0], [10, 100, 0], [0, 1, 10]]], edges=(None, None)
-)
+PdMatrixChol3 = Domain([np.eye(3), [[0.1, 0, 0], [10, 100, 0], [0, 1, 10]]], edges=(None, None))
 
 
 def PdMatrixChol(n):
@@ -537,9 +529,7 @@ class TestMatchesScipy(SeededTest):
 
         self.check_logp(model, value, domain, paramdomains, logp, decimal=decimal)
 
-    def check_logp(
-        self, model, value, domain, paramdomains, logp_reference, decimal=None
-    ):
+    def check_logp(self, model, value, domain, paramdomains, logp_reference, decimal=None):
         domains = paramdomains.copy()
         domains["value"] = domain
         logp = model.fastlogp
@@ -547,9 +537,7 @@ class TestMatchesScipy(SeededTest):
             pt = Point(pt, model=model)
             if decimal is None:
                 decimal = select_by_precision(float64=6, float32=3)
-            assert_almost_equal(
-                logp(pt), logp_reference(pt), decimal=decimal, err_msg=str(pt)
-            )
+            assert_almost_equal(logp(pt), logp_reference(pt), decimal=decimal, err_msg=str(pt))
 
     def check_logcdf(
         self,
@@ -614,17 +602,13 @@ class TestMatchesScipy(SeededTest):
             Triangular,
             Runif,
             {"lower": -Rplusunif, "c": Runif, "upper": Rplusunif},
-            lambda value, c, lower, upper: sp.triang.logpdf(
-                value, c - lower, lower, upper - lower
-            ),
+            lambda value, c, lower, upper: sp.triang.logpdf(value, c - lower, lower, upper - lower),
         )
         self.check_logcdf(
             Triangular,
             Runif,
             {"lower": -Rplusunif, "c": Runif, "upper": Rplusunif},
-            lambda value, c, lower, upper: sp.triang.logcdf(
-                value, c - lower, lower, upper - lower
-            ),
+            lambda value, c, lower, upper: sp.triang.logcdf(value, c - lower, lower, upper - lower),
         )
 
     def test_bound_normal(self):
@@ -773,9 +757,7 @@ class TestMatchesScipy(SeededTest):
             {"alpha": Rplus, "beta": Rplus},
             lambda value, alpha, beta: sp.beta.logpdf(value, alpha, beta),
         )
-        self.pymc3_matches_scipy(
-            Beta, Unit, {"mu": Unit, "sigma": Rplus}, beta_mu_sigma
-        )
+        self.pymc3_matches_scipy(Beta, Unit, {"mu": Unit, "sigma": Rplus}, beta_mu_sigma)
         self.check_logcdf(
             Beta,
             Unit,
@@ -787,15 +769,10 @@ class TestMatchesScipy(SeededTest):
         # Scipy does not have a built-in Kumaraswamy pdf
         def scipy_log_pdf(value, a, b):
             return (
-                np.log(a)
-                + np.log(b)
-                + (a - 1) * np.log(value)
-                + (b - 1) * np.log(1 - value ** a)
+                np.log(a) + np.log(b) + (a - 1) * np.log(value) + (b - 1) * np.log(1 - value ** a)
             )
 
-        self.pymc3_matches_scipy(
-            Kumaraswamy, Unit, {"a": Rplus, "b": Rplus}, scipy_log_pdf
-        )
+        self.pymc3_matches_scipy(Kumaraswamy, Unit, {"a": Rplus, "b": Rplus}, scipy_log_pdf)
 
     def test_exponential(self):
         self.pymc3_matches_scipy(
@@ -816,14 +793,46 @@ class TestMatchesScipy(SeededTest):
             Geometric, Nat, {"p": Unit}, lambda value, p: np.log(sp.geom.pmf(value, p))
         )
 
+    def test_hypergeometric(self):
+        self.pymc3_matches_scipy(
+            HyperGeometric,
+            Nat,
+            {"N": NatSmall, "k": NatSmall, "n": NatSmall},
+            lambda value, N, k, n: sp.hypergeom.logpmf(value, N, k, n),
+        )
+
     def test_negative_binomial(self):
         def test_fun(value, mu, alpha):
             return sp.nbinom.logpmf(value, alpha, 1 - mu / (mu + alpha))
 
+        self.pymc3_matches_scipy(NegativeBinomial, Nat, {"mu": Rplus, "alpha": Rplus}, test_fun)
         self.pymc3_matches_scipy(
-            NegativeBinomial, Nat, {"mu": Rplus, "alpha": Rplus}, test_fun
+            NegativeBinomial,
+            Nat,
+            {"p": Unit, "n": Rplus},
+            lambda value, p, n: sp.nbinom.logpmf(value, n, p),
         )
 
+    @pytest.mark.parametrize(
+        "mu, p, alpha, n, expected",
+        [
+            (5, None, None, None, "Must specify either alpha or n."),
+            (None, 0.5, None, None, "Must specify either alpha or n."),
+            (None, None, None, None, "Must specify either alpha or n."),
+            (5, None, 2, 2, "Can't specify both alpha and n."),
+            (None, 0.5, 2, 2, "Can't specify both alpha and n."),
+            (None, None, 2, 2, "Can't specify both alpha and n."),
+            (None, None, 2, None, "Must specify either mu or p."),
+            (None, None, None, 2, "Must specify either mu or p."),
+            (5, 0.5, 2, None, "Can't specify both mu and p."),
+            (5, 0.5, None, 2, "Can't specify both mu and p."),
+        ],
+    )
+    def test_negative_binomial_init_fail(self, mu, p, alpha, n, expected):
+        with Model():
+            with pytest.raises(ValueError, match=f"Incompatible parametrization. {expected}"):
+                NegativeBinomial("x", mu=mu, p=p, alpha=alpha, n=n)
+
     def test_laplace(self):
         self.pymc3_matches_scipy(
             Laplace,
@@ -843,9 +852,7 @@ class TestMatchesScipy(SeededTest):
             Lognormal,
             Rplus,
             {"mu": R, "tau": Rplusbig},
-            lambda value, mu, tau: floatX(
-                sp.lognorm.logpdf(value, tau ** -0.5, 0, np.exp(mu))
-            ),
+            lambda value, mu, tau: floatX(sp.lognorm.logpdf(value, tau ** -0.5, 0, np.exp(mu))),
         )
         self.check_logcdf(
             Lognormal,
@@ -906,13 +913,9 @@ class TestMatchesScipy(SeededTest):
         )
 
         def test_fun(value, mu, sigma):
-            return sp.gamma.logpdf(
-                value, mu ** 2 / sigma ** 2, scale=1.0 / (mu / sigma ** 2)
-            )
+            return sp.gamma.logpdf(value, mu ** 2 / sigma ** 2, scale=1.0 / (mu / sigma ** 2))
 
-        self.pymc3_matches_scipy(
-            Gamma, Rplus, {"mu": Rplusbig, "sigma": Rplusbig}, test_fun
-        )
+        self.pymc3_matches_scipy(Gamma, Rplus, {"mu": Rplusbig, "sigma": Rplusbig}, test_fun)
 
         self.check_logcdf(
             Gamma,
@@ -921,6 +924,10 @@ class TestMatchesScipy(SeededTest):
             lambda value, alpha, beta: sp.gamma.logcdf(value, alpha, scale=1.0 / beta),
         )
 
+    @pytest.mark.xfail(
+        condition=(theano.config.floatX == "float32"),
+        reason="Fails on float32 due to numerical issues",
+    )
     def test_inverse_gamma(self):
         self.pymc3_matches_scipy(
             InverseGamma,
@@ -928,6 +935,12 @@ class TestMatchesScipy(SeededTest):
             {"alpha": Rplus, "beta": Rplus},
             lambda value, alpha, beta: sp.invgamma.logpdf(value, alpha, scale=beta),
         )
+        self.check_logcdf(
+            InverseGamma,
+            Rplus,
+            {"alpha": Rplus, "beta": Rplus},
+            lambda value, alpha, beta: sp.invgamma.logcdf(value, alpha, scale=beta),
+        )
 
     @pytest.mark.xfail(
         condition=(theano.config.floatX == "float32"),
@@ -938,9 +951,7 @@ class TestMatchesScipy(SeededTest):
             alpha, beta = InverseGamma._get_alpha_beta(None, None, mu, sigma)
             return sp.invgamma.logpdf(value, alpha, scale=beta)
 
-        self.pymc3_matches_scipy(
-            InverseGamma, Rplus, {"mu": Rplus, "sigma": Rplus}, test_fun
-        )
+        self.pymc3_matches_scipy(InverseGamma, Rplus, {"mu": Rplus, "sigma": Rplus}, test_fun)
 
     def test_pareto(self):
         self.pymc3_matches_scipy(
@@ -1000,9 +1011,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_beta_binomial(self):
         self.checkd(BetaBinomial, Nat, {"alpha": Rplus, "beta": Rplus, "n": NatSmall})
 
@@ -1011,9 +1020,7 @@ class TestMatchesScipy(SeededTest):
             Bernoulli,
             Bool,
             {"logit_p": R},
-            lambda value, logit_p: sp.bernoulli.logpmf(
-                value, scipy.special.expit(logit_p)
-            ),
+            lambda value, logit_p: sp.bernoulli.logpmf(value, scipy.special.expit(logit_p)),
         )
         self.pymc3_matches_scipy(
             Bernoulli, Bool, {"p": Unit}, lambda value, p: sp.bernoulli.logpmf(value, p)
@@ -1046,21 +1053,15 @@ class TestMatchesScipy(SeededTest):
         assert np.isinf(x.logp({"x": 0}))
 
     def test_constantdist(self):
-        self.pymc3_matches_scipy(
-            Constant, I, {"c": I}, lambda value, c: np.log(c == value)
-        )
+        self.pymc3_matches_scipy(Constant, I, {"c": I}, lambda value, c: np.log(c == value))
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_zeroinflatedpoisson(self):
         self.checkd(ZeroInflatedPoisson, Nat, {"theta": Rplus, "psi": Unit})
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_zeroinflatednegativebinomial(self):
         self.checkd(
             ZeroInflatedNegativeBinomial,
@@ -1069,9 +1070,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_zeroinflatedbinomial(self):
         self.checkd(ZeroInflatedBinomial, Nat, {"n": NatSmall, "p": Unit, "psi": Unit})
 
@@ -1297,9 +1296,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_AR1(self, n):
-        self.pymc3_matches_scipy(
-            AR1, Vector(R, n), {"k": Unit, "tau_e": Rplus}, AR1_logpdf
-        )
+        self.pymc3_matches_scipy(AR1, Vector(R, n), {"k": Unit, "tau_e": Rplus}, AR1_logpdf)
 
     @pytest.mark.parametrize("n", [2, 3])
     def test_wishart(self, n):
@@ -1324,9 +1321,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3])
     def test_dirichlet(self, n):
-        self.pymc3_matches_scipy(
-            Dirichlet, Simplex(n), {"a": Vector(Rplus, n)}, dirichlet_logpdf
-        )
+        self.pymc3_matches_scipy(Dirichlet, Simplex(n), {"a": Vector(Rplus, n)}, dirichlet_logpdf)
 
     def test_dirichlet_shape(self):
         a = tt.as_tensor_variable(np.r_[1, 2])
@@ -1473,6 +1468,28 @@ class TestMatchesScipy(SeededTest):
             decimal=4,
         )
 
+    def test_batch_multinomial(self):
+        n = 10
+        vals = np.zeros((4, 5, 3), dtype="int32")
+        p = np.zeros_like(vals, dtype=theano.config.floatX)
+        inds = np.random.randint(vals.shape[-1], size=vals.shape[:-1])[..., None]
+        np.put_along_axis(vals, inds, n, axis=-1)
+        np.put_along_axis(p, inds, 1, axis=-1)
+
+        dist = Multinomial.dist(n=n, p=p, shape=vals.shape)
+        value = tt.tensor3(dtype="int32")
+        value.tag.test_value = np.zeros_like(vals, dtype="int32")
+        logp = tt.exp(dist.logp(value))
+        f = theano.function(inputs=[value], outputs=logp)
+        assert_almost_equal(
+            f(vals),
+            np.ones(vals.shape[:-1] + (1,)),
+            decimal=select_by_precision(float64=6, float32=3),
+        )
+
+        sample = dist.random(size=2)
+        assert_allclose(sample, np.stack([vals, vals], axis=0))
+
     def test_categorical_bounds(self):
         with Model():
             x = Categorical("x", p=np.array([0.2, 0.3, 0.5]))
@@ -1528,9 +1545,7 @@ class TestMatchesScipy(SeededTest):
 
     def test_get_tau_sigma(self):
         sigma = np.array([2])
-        assert_almost_equal(
-            continuous.get_tau_sigma(sigma=sigma), [1.0 / sigma ** 2, sigma]
-        )
+        assert_almost_equal(continuous.get_tau_sigma(sigma=sigma), [1.0 / sigma ** 2, sigma])
 
     @pytest.mark.parametrize(
         "value,mu,sigma,nu,logp",
@@ -1581,9 +1596,7 @@ class TestMatchesScipy(SeededTest):
             err_msg=str((value, mu, sigma, nu, logcdf)),
         )
 
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_vonmises(self):
         self.pymc3_matches_scipy(
             VonMises,
@@ -1625,8 +1638,7 @@ class TestMatchesScipy(SeededTest):
             Unit,
             {"mu": R, "sigma": Rplus},
             lambda value, mu, sigma: (
-                sp.norm.logpdf(logit(value), mu, sigma)
-                - (np.log(value) + np.log1p(-value))
+                sp.norm.logpdf(logit(value), mu, sigma) - (np.log(value) + np.log1p(-value))
             ),
             decimal=select_by_precision(float64=6, float32=1),
         )
@@ -1640,9 +1652,7 @@ class TestMatchesScipy(SeededTest):
             Rice,
             Rplus,
             {"nu": Rplus, "sigma": Rplusbig},
-            lambda value, nu, sigma: sp.rice.logpdf(
-                value, b=nu / sigma, loc=0, scale=sigma
-            ),
+            lambda value, nu, sigma: sp.rice.logpdf(value, b=nu / sigma, loc=0, scale=sigma),
         )
         self.pymc3_matches_scipy(
             Rice,
@@ -1651,9 +1661,7 @@ class TestMatchesScipy(SeededTest):
             lambda value, b, sigma: sp.rice.logpdf(value, b=b, loc=0, scale=sigma),
         )
 
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_moyal(self):
         self.pymc3_matches_scipy(
             Moyal,
@@ -1668,9 +1676,7 @@ class TestMatchesScipy(SeededTest):
             lambda value, mu, sigma: floatX(sp.moyal.logcdf(value, mu, sigma)),
         )
 
-    @pytest.mark.xfail(
-        condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-    )
+    @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_interpolated(self):
         for mu in R.vals:
             for sigma in Rplus.vals:
@@ -1682,9 +1688,7 @@ class TestMatchesScipy(SeededTest):
                     def __init__(self, **kwargs):
                         x_points = np.linspace(xmin, xmax, 100000)
                         pdf_points = sp.norm.pdf(x_points, loc=mu, scale=sigma)
-                        super().__init__(
-                            x_points=x_points, pdf_points=pdf_points, **kwargs
-                        )
+                        super().__init__(x_points=x_points, pdf_points=pdf_points, **kwargs)
 
                 def ref_pdf(value):
                     return np.where(
@@ -1770,7 +1774,7 @@ def test_bound():
         BoundPoissonPositionalArgs = Bound(Poisson, upper=6)("x", 2.0)
 
 
-class TestLatex:
+class TestStrAndLatexRepr:
     def setup_class(self):
         # True parameter values
         alpha, sigma = 1, 1
@@ -1793,36 +1797,123 @@ class TestLatex:
             # Test Cholesky parameterization
             Z = MvNormal("Z", mu=np.zeros(2), chol=np.eye(2), shape=(2,))
 
+            # NegativeBinomial representations to test issue 4186
+            nb1 = pm.NegativeBinomial(
+                "nb_with_mu_alpha", mu=pm.Normal("nbmu"), alpha=pm.Gamma("nbalpha", mu=6, sigma=1)
+            )
+            nb2 = pm.NegativeBinomial("nb_with_p_n", p=pm.Uniform("nbp"), n=10)
+
             # Expected value of outcome
             mu = Deterministic("mu", floatX(alpha + tt.dot(X, b)))
 
+            # add a bounded variable as well
+            bound_var = Bound(Normal, lower=1.0)("bound_var", mu=0, sigma=10)
+
+            # KroneckerNormal
+            n, m = 3, 4
+            covs = [np.eye(n), np.eye(m)]
+            kron_normal = KroneckerNormal("kron_normal", mu=np.zeros(n * m), covs=covs, shape=n * m)
+
+            # MatrixNormal
+            matrix_normal = MatrixNormal(
+                "mat_normal",
+                mu=np.random.normal(size=n),
+                rowcov=np.eye(n),
+                colchol=np.linalg.cholesky(np.eye(n)),
+                shape=(n, n),
+            )
+
             # Likelihood (sampling distribution) of observations
             Y_obs = Normal("Y_obs", mu=mu, sigma=sigma, observed=Y)
-        self.distributions = [alpha, sigma, mu, b, Z, Y_obs]
-        self.expected = (
-            r"$\text{alpha} \sim \text{Normal}(\mathit{mu}=0.0,~\mathit{sigma}=10.0)$",
-            r"$\text{sigma} \sim \text{HalfNormal}(\mathit{sigma}=1.0)$",
-            r"$\text{mu} \sim \text{Deterministic}(\text{alpha},~\text{Constant},~\text{beta})$",
-            r"$\text{beta} \sim \text{Normal}(\mathit{mu}=0.0,~\mathit{sigma}=10.0)$",
-            r"$Z \sim \text{MvNormal}(\mathit{mu}=array, \mathit{chol}=array)$",
-            r"$\text{Y_obs} \sim \text{Normal}(\mathit{mu}=\text{mu},~\mathit{sigma}=f(\text{sigma}))$",
-        )
+
+        self.distributions = [alpha, sigma, mu, b, Z, nb1, nb2, Y_obs, bound_var]
+        self.expected = {
+            "latex": (
+                r"$\text{alpha} \sim \text{Normal}$",
+                r"$\text{sigma} \sim \text{HalfNormal}$",
+                r"$\text{mu} \sim \text{Deterministic}$",
+                r"$\text{beta} \sim \text{Normal}$",
+                r"$\text{Z} \sim \text{MvNormal}$",
+                r"$\text{nb_with_mu_alpha} \sim \text{NegativeBinomial}$",
+                r"$\text{nb_with_p_n} \sim \text{NegativeBinomial}$",
+                r"$\text{Y_obs} \sim \text{Normal}$",
+                r"$\text{bound_var} \sim \text{Bound}$ -- \text{Normal}$",
+                r"$\text{kron_normal} \sim \text{KroneckerNormal}$",
+                r"$\text{mat_normal} \sim \text{MatrixNormal}$",
+            ),
+            "plain": (
+                r"alpha ~ Normal",
+                r"sigma ~ HalfNormal",
+                r"mu ~ Deterministic",
+                r"beta ~ Normal",
+                r"Z ~ MvNormal",
+                r"nb_with_mu_alpha ~ NegativeBinomial",
+                r"nb_with_p_n ~ NegativeBinomial",
+                r"Y_obs ~ Normal",
+                r"bound_var ~ Bound-Normal",
+                r"kron_normal ~ KroneckerNormal",
+                r"mat_normal ~ MatrixNormal",
+            ),
+            "latex_with_params": (
+                r"$\text{alpha} \sim \text{Normal}(\mathit{mu}=0.0,~\mathit{sigma}=10.0)$",
+                r"$\text{sigma} \sim \text{HalfNormal}(\mathit{sigma}=1.0)$",
+                r"$\text{mu} \sim \text{Deterministic}(\text{alpha},~\text{Constant},~\text{beta})$",
+                r"$\text{beta} \sim \text{Normal}(\mathit{mu}=0.0,~\mathit{sigma}=10.0)$",
+                r"$\text{Z} \sim \text{MvNormal}(\mathit{mu}=array,~\mathit{chol_cov}=array)$",
+                r"$\text{nb_with_mu_alpha} \sim \text{NegativeBinomial}(\mathit{mu}=\text{nbmu},~\mathit{alpha}=\text{nbalpha})$",
+                r"$\text{nb_with_p_n} \sim \text{NegativeBinomial}(\mathit{p}=\text{nbp},~\mathit{n}=10)$",
+                r"$\text{Y_obs} \sim \text{Normal}(\mathit{mu}=\text{mu},~\mathit{sigma}=f(\text{sigma}))$",
+                r"$\text{bound_var} \sim \text{Bound}(\mathit{lower}=1.0,~\mathit{upper}=\text{None})$ -- \text{Normal}(\mathit{mu}=0.0,~\mathit{sigma}=10.0)$",
+                r"$\text{kron_normal} \sim \text{KroneckerNormal}(\mathit{mu}=array)$",
+                r"$\text{mat_normal} \sim \text{MatrixNormal}(\mathit{mu}=array,~\mathit{rowcov}=array,~\mathit{colchol_cov}=array)$",
+            ),
+            "plain_with_params": (
+                r"alpha ~ Normal(mu=0.0, sigma=10.0)",
+                r"sigma ~ HalfNormal(sigma=1.0)",
+                r"mu ~ Deterministic(alpha, Constant, beta)",
+                r"beta ~ Normal(mu=0.0, sigma=10.0)",
+                r"Z ~ MvNormal(mu=array, chol_cov=array)",
+                r"nb_with_mu_alpha ~ NegativeBinomial(mu=nbmu, alpha=nbalpha)",
+                r"nb_with_p_n ~ NegativeBinomial(p=nbp, n=10)",
+                r"Y_obs ~ Normal(mu=mu, sigma=f(sigma))",
+                r"bound_var ~ Bound(lower=1.0, upper=None)-Normal(mu=0.0, sigma=10.0)",
+                r"kron_normal ~ KroneckerNormal(mu=array)",
+                r"mat_normal ~ MatrixNormal(mu=array, rowcov=array, colchol_cov=array)",
+            ),
+        }
 
     def test__repr_latex_(self):
-        for distribution, tex in zip(self.distributions, self.expected):
+        for distribution, tex in zip(self.distributions, self.expected["latex_with_params"]):
             assert distribution._repr_latex_() == tex
 
         model_tex = self.model._repr_latex_()
 
-        for tex in self.expected:  # make sure each variable is in the model
+        # make sure each variable is in the model
+        for tex in self.expected["latex"]:
             for segment in tex.strip("$").split(r"\sim"):
                 assert segment in model_tex
 
     def test___latex__(self):
-        for distribution, tex in zip(self.distributions, self.expected):
+        for distribution, tex in zip(self.distributions, self.expected["latex_with_params"]):
             assert distribution._repr_latex_() == distribution.__latex__()
         assert self.model._repr_latex_() == self.model.__latex__()
 
+    def test___str__(self):
+        for distribution, str_repr in zip(self.distributions, self.expected["plain"]):
+            assert distribution.__str__() == str_repr
+
+        model_str = self.model.__str__()
+        for str_repr in self.expected["plain"]:
+            assert str_repr in model_str
+
+    def test_str(self):
+        for distribution, str_repr in zip(self.distributions, self.expected["plain"]):
+            assert str(distribution) == str_repr
+
+        model_str = str(self.model)
+        for str_repr in self.expected["plain"]:
+            assert str_repr in model_str
+
 
 def test_discrete_trafo():
     with pytest.raises(ValueError) as err:
@@ -1872,3 +1963,17 @@ class TestBugfixes:
         assert isinstance(actual_a, np.ndarray)
         assert actual_a.shape == (X.shape[0],)
         pass
+
+
+def test_serialize_density_dist():
+    def func(x):
+        return -2 * (x ** 2).sum()
+
+    with pm.Model():
+        pm.Normal("x")
+        y = pm.DensityDist("y", func)
+        pm.sample(draws=5, tune=1, mp_ctx="spawn")
+
+    import pickle
+
+    pickle.loads(pickle.dumps(y))
