@@ -12,32 +12,29 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
+import unittest.mock as mock
+
+from contextlib import ExitStack as does_not_raise
 from itertools import combinations
-import packaging
 from typing import Tuple
-import numpy as np
 
-try:
-    import unittest.mock as mock  # py3
-except ImportError:
-    import mock
-
-import numpy.testing as npt
 import arviz as az
-import pymc3 as pm
-import theano.tensor as tt
-from theano import shared
-import theano
-from pymc3.tests.models import simple_init
-from pymc3.tests.helpers import SeededTest
-from pymc3.exceptions import IncorrectArgumentsError
-from scipy import stats
+import numpy as np
+import numpy.testing as npt
 import pytest
+import theano
+import theano.tensor as tt
+
+from scipy import stats
+from theano import shared
+
+import pymc3 as pm
+
+from pymc3.exceptions import IncorrectArgumentsError, SamplingError
+from pymc3.tests.helpers import SeededTest
+from pymc3.tests.models import simple_init
 
 
-@pytest.mark.xfail(
-    condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-)
 @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
 class TestSample(SeededTest):
     def setup_method(self):
@@ -159,31 +156,38 @@ class TestSample(SeededTest):
     def test_trace_report(self, step_cls, discard):
         with self.model:
             # add more variables, because stats are 2D with CompoundStep!
-            pm.Uniform('uni')
+            pm.Uniform("uni")
             trace = pm.sample(
-                draws=100, tune=50, cores=1,
-                discard_tuned_samples=discard,
-                step=step_cls()
+                draws=100, tune=50, cores=1, discard_tuned_samples=discard, step=step_cls()
             )
             assert trace.report.n_tune == 50
             assert trace.report.n_draws == 100
             assert isinstance(trace.report.t_sampling, float)
         pass
 
-    def test_return_inferencedata(self):
+    def test_trace_report_bart(self):
+        X = np.random.normal(0, 1, size=(3, 250)).T
+        Y = np.random.normal(0, 1, size=250)
+        X[:, 0] = np.random.normal(Y, 0.1)
+
+        with pm.Model() as model:
+            mu = pm.BART("mu", X, Y, m=20)
+            sigma = pm.HalfNormal("sigma", 1)
+            y = pm.Normal("y", mu, sigma, observed=Y)
+            trace = pm.sample(500, tune=100, random_seed=3415)
+        var_imp = trace.report.variable_importance
+        assert var_imp[0] > var_imp[1:].sum()
+        npt.assert_almost_equal(var_imp.sum(), 1)
+
+    def test_return_inferencedata(self, monkeypatch):
         with self.model:
-            kwargs = dict(
-                draws=100, tune=50, cores=1,
-                chains=2, step=pm.Metropolis()
-            )
-            v = packaging.version.parse(pm.__version__)
-            if v.major > 3 or v.minor >= 10:
-                with pytest.warns(FutureWarning, match="pass return_inferencedata"):
-                    result = pm.sample(**kwargs)
+            kwargs = dict(draws=100, tune=50, cores=1, chains=2, step=pm.Metropolis())
 
             # trace with tuning
             with pytest.warns(UserWarning, match="will be included"):
-                result = pm.sample(**kwargs, return_inferencedata=False, discard_tuned_samples=False)
+                result = pm.sample(
+                    **kwargs, return_inferencedata=False, discard_tuned_samples=False
+                )
             assert isinstance(result, pm.backends.base.MultiTrace)
             assert len(result) == 150
 
@@ -194,22 +198,33 @@ class TestSample(SeededTest):
             assert result.posterior.sizes["chain"] == 2
             assert len(result._groups_warmup) > 0
 
-            # inferencedata without tuning
-            result = pm.sample(**kwargs, return_inferencedata=True, discard_tuned_samples=True)
+            # inferencedata without tuning, with idata_kwargs
+            prior = pm.sample_prior_predictive()
+            result = pm.sample(
+                **kwargs,
+                return_inferencedata=True,
+                discard_tuned_samples=True,
+                idata_kwargs={"prior": prior},
+                random_seed=-1
+            )
+            assert "prior" in result
             assert isinstance(result, az.InferenceData)
             assert result.posterior.sizes["draw"] == 100
             assert result.posterior.sizes["chain"] == 2
             assert len(result._groups_warmup) == 0
+
+            # check warning for version 3.10 onwards
+            monkeypatch.setattr("pymc3.__version__", "3.10")
+            with pytest.warns(FutureWarning, match="pass return_inferencedata"):
+                result = pm.sample(**kwargs)
         pass
 
-    @pytest.mark.parametrize('cores', [1, 2])
+    @pytest.mark.parametrize("cores", [1, 2])
     def test_sampler_stat_tune(self, cores):
         with self.model:
             tune_stat = pm.sample(
-                tune=5, draws=7, cores=cores,
-                discard_tuned_samples=False,
-                step=pm.Metropolis()
-            ).get_sampler_stats('tune', chains=1)
+                tune=5, draws=7, cores=cores, discard_tuned_samples=False, step=pm.Metropolis()
+            ).get_sampler_stats("tune", chains=1)
             assert list(tune_stat).count(True) == 5
             assert list(tune_stat).count(False) == 7
         pass
@@ -227,9 +242,7 @@ class TestSample(SeededTest):
         with pytest.raises(error):
             pm.sampling._check_start_shape(self.model, start)
 
-    @pytest.mark.parametrize(
-        "start", [{"x": np.array([1, 1])}, {"x": [10, 10]}, {"x": [-10, -10]}]
-    )
+    @pytest.mark.parametrize("start", [{"x": np.array([1, 1])}, {"x": [10, 10]}, {"x": [-10, -10]}])
     def test_sample_start_good_shape(self, start):
         pm.sampling._check_start_shape(self.model, start)
 
@@ -241,8 +254,13 @@ class TestSample(SeededTest):
             for cores in test_cores:
                 for chain in test_chains:
                     pm.sample(
-                        10, tune=0, chains=chain, step=self.step, cores=cores, random_seed=self.random_seed,
-                        callback=callback
+                        10,
+                        tune=0,
+                        chains=chain,
+                        step=self.step,
+                        cores=cores,
+                        random_seed=self.random_seed,
+                        callback=callback,
                     )
                     assert callback.called
 
@@ -255,9 +273,14 @@ class TestSample(SeededTest):
 
         with self.model:
             trace = pm.sample(
-                        10, tune=0, chains=1, step=self.step, cores=1, random_seed=self.random_seed,
-                        callback=callback
-                    )
+                10,
+                tune=0,
+                chains=1,
+                step=self.step,
+                cores=1,
+                random_seed=self.random_seed,
+                callback=callback,
+            )
             assert len(trace) == trace_cancel_length
 
 
@@ -276,9 +299,7 @@ def test_partial_trace_sample():
         trace = pm.sample(trace=[a])
 
 
-@pytest.mark.xfail(
-    condition=(theano.config.floatX == "float32"), reason="Fails on float32"
-)
+@pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
 class TestNamedSampling(SeededTest):
     def test_shared_named(self):
         G_var = shared(value=np.atleast_2d(1.0), broadcastable=(True, False), name="G")
@@ -342,16 +363,6 @@ class TestChooseBackend:
             pm.sampling._choose_backend(["var1", "var2"], "chain")
         nd.assert_called_with(vars=["var1", "var2"])
 
-    def test_choose_backend_invalid(self):
-        with pytest.raises(ValueError):
-            pm.sampling._choose_backend("invalid", "chain")
-
-    def test_choose_backend_shortcut(self):
-        backend = mock.Mock()
-        shortcuts = {"test_backend": {"backend": backend, "name": None}}
-        pm.sampling._choose_backend("test_backend", "chain", shortcuts=shortcuts)
-        assert backend.called
-
 
 class TestSamplePPC(SeededTest):
     def test_normal_scalar(self):
@@ -404,7 +415,6 @@ class TestSamplePPC(SeededTest):
             _, pval = stats.kstest(ppc["a"] - trace["mu"], stats.norm(loc=0, scale=1).cdf)
             assert pval > 0.001
 
-
         # size argument not introduced to fast version [2019/08/20:rpg]
         with model:
             ppc = pm.sample_posterior_predictive(trace, size=5, var_names=["a"])
@@ -460,7 +470,6 @@ class TestSamplePPC(SeededTest):
             assert "a" in ppc
             assert ppc["a"].shape == (12, 2)
 
-
             # size unsupported by fast_ version  argument. [2019/08/19:rpg]
             ppc = pm.sample_posterior_predictive(trace, samples=10, var_names=["a"], size=4)
             assert "a" in ppc
@@ -484,7 +493,7 @@ class TestSamplePPC(SeededTest):
             with pytest.raises(IncorrectArgumentsError):
                 ppc = pm.sample_posterior_predictive(trace, vars=[a], var_names=["a"])
             # test wrong type argument
-            bad_trace = {'mu': stats.norm.rvs(size=1000)}
+            bad_trace = {"mu": stats.norm.rvs(size=1000)}
             with pytest.raises(TypeError):
                 ppc = pm.sample_posterior_predictive(bad_trace)
             with pytest.raises(TypeError):
@@ -518,7 +527,6 @@ class TestSamplePPC(SeededTest):
             assert "a" in ppc
             assert ppc["a"].shape == (12, 2)
 
-
     def test_sum_normal(self):
         with pm.Model() as model:
             a = pm.Normal("a", sigma=0.2)
@@ -546,8 +554,6 @@ class TestSamplePPC(SeededTest):
             _, pval = stats.kstest(ppc["b"], stats.norm(scale=scale).cdf)
             assert pval > 0.001
 
-
-
     def test_model_not_drawable_prior(self):
         data = np.random.poisson(lam=10, size=200)
         model = pm.Model()
@@ -566,7 +572,6 @@ class TestSamplePPC(SeededTest):
             samples = pm.fast_sample_posterior_predictive(trace, 40)
             assert samples["foo"].shape == (40, 200)
 
-
     def test_model_shared_variable(self):
         x = np.random.randn(100)
         y = x > 0
@@ -584,14 +589,13 @@ class TestSamplePPC(SeededTest):
 
         samples = 100
         with model:
-            post_pred = pm.sample_posterior_predictive(trace, samples=samples, var_names=["p", "obs"])
+            post_pred = pm.sample_posterior_predictive(
+                trace, samples=samples, var_names=["p", "obs"]
+            )
 
-        expected_p = np.array(
-            [logistic.eval({coeff: val}) for val in trace["x"][:samples]]
-        )
+        expected_p = np.array([logistic.eval({coeff: val}) for val in trace["x"][:samples]])
         assert post_pred["obs"].shape == (samples, 3)
-        assert np.allclose(post_pred["p"], expected_p)
-
+        npt.assert_allclose(post_pred["p"], expected_p)
 
         # fast version
         samples = 100
@@ -600,15 +604,14 @@ class TestSamplePPC(SeededTest):
                 trace, samples=samples, var_names=["p", "obs"]
             )
 
-        expected_p = np.array(
-            [logistic.eval({coeff: val}) for val in trace["x"][:samples]]
-        )
+        expected_p = np.array([logistic.eval({coeff: val}) for val in trace["x"][:samples]])
         assert post_pred["obs"].shape == (samples, 3)
-        assert np.allclose(post_pred["p"], expected_p)
+        npt.assert_allclose(post_pred["p"], expected_p)
 
     def test_deterministic_of_observed(self):
-        meas_in_1 = pm.theanof.floatX(2 + 4 * np.random.randn(100))
-        meas_in_2 = pm.theanof.floatX(5 + 4 * np.random.randn(100))
+        meas_in_1 = pm.theanof.floatX(2 + 4 * np.random.randn(10))
+        meas_in_2 = pm.theanof.floatX(5 + 4 * np.random.randn(10))
+        nchains = 2
         with pm.Model() as model:
             mu_in_1 = pm.Normal("mu_in_1", 0, 1)
             sigma_in_1 = pm.HalfNormal("sd_in_1", 1)
@@ -620,43 +623,38 @@ class TestSamplePPC(SeededTest):
             out_diff = in_1 + in_2
             pm.Deterministic("out", out_diff)
 
-            trace = pm.sample(100)
-            ppc_trace = pm.trace_to_dataframe(
-                trace, varnames=[n for n in trace.varnames if n != "out"]
-            ).to_dict("records")
+            trace = pm.sample(100, chains=nchains)
+            np.random.seed(0)
             with pytest.warns(DeprecationWarning):
                 ppc = pm.sample_posterior_predictive(
                     model=model,
-                    trace=ppc_trace,
-                    samples=len(ppc_trace),
+                    trace=trace,
+                    samples=len(trace) * nchains,
                     vars=(model.deterministics + model.basic_RVs),
                 )
 
-            rtol = 1e-5 if theano.config.floatX == "float64" else 1e-3
-            assert np.allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
+            rtol = 1e-5 if theano.config.floatX == "float64" else 1e-4
+            npt.assert_allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
 
+            np.random.seed(0)
             ppc = pm.sample_posterior_predictive(
                 model=model,
-                trace=ppc_trace,
-                samples=len(ppc_trace),
-                var_names = [var.name for var in (model.deterministics + model.basic_RVs)]
+                trace=trace,
+                samples=len(trace) * nchains,
+                var_names=[var.name for var in (model.deterministics + model.basic_RVs)],
             )
 
-            rtol = 1e-5 if theano.config.floatX == "float64" else 1e-3
-            assert np.allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
-
+            npt.assert_allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
 
+            np.random.seed(0)
             ppc = pm.fast_sample_posterior_predictive(
                 model=model,
-                trace=ppc_trace,
-                samples=len(ppc_trace),
-                var_names = [var.name for var in (model.deterministics + model.basic_RVs)]
+                trace=trace,
+                samples=len(trace) * nchains,
+                var_names=[var.name for var in (model.deterministics + model.basic_RVs)],
             )
 
-            rtol = 1e-5 if theano.config.floatX == "float64" else 1e-3
-            assert np.allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
-
-
+            npt.assert_allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
 
     def test_deterministic_of_observed_modified_interface(self):
         meas_in_1 = pm.theanof.floatX(2 + 4 * np.random.randn(100))
@@ -684,7 +682,7 @@ class TestSamplePPC(SeededTest):
             )
 
             rtol = 1e-5 if theano.config.floatX == "float64" else 1e-3
-            assert np.allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
+            npt.assert_allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
 
             ppc = pm.fast_sample_posterior_predictive(
                 model=model,
@@ -694,8 +692,7 @@ class TestSamplePPC(SeededTest):
             )
 
             rtol = 1e-5 if theano.config.floatX == "float64" else 1e-3
-            assert np.allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
-
+            npt.assert_allclose(ppc["in_1"] + ppc["in_2"], ppc["out"], rtol=rtol)
 
     def test_variable_type(self):
         with pm.Model() as model:
@@ -712,27 +709,53 @@ class TestSamplePPC(SeededTest):
 
 class TestSamplePPCW(SeededTest):
     def test_sample_posterior_predictive_w(self):
-        data0 = np.random.normal(0, 1, size=500)
+        data0 = np.random.normal(0, 1, size=50)
+        warning_msg = "The number of samples is too small to check convergence reliably"
 
         with pm.Model() as model_0:
             mu = pm.Normal("mu", mu=0, sigma=1)
             y = pm.Normal("y", mu=mu, sigma=1, observed=data0)
-            trace_0 = pm.sample()
+            with pytest.warns(UserWarning, match=warning_msg):
+                trace_0 = pm.sample(10, tune=0, chains=2, return_inferencedata=False)
+            idata_0 = az.from_pymc3(trace_0)
 
         with pm.Model() as model_1:
             mu = pm.Normal("mu", mu=0, sigma=1, shape=len(data0))
             y = pm.Normal("y", mu=mu, sigma=1, observed=data0)
-            trace_1 = pm.sample()
+            with pytest.warns(UserWarning, match=warning_msg):
+                trace_1 = pm.sample(10, tune=0, chains=2, return_inferencedata=False)
+            idata_1 = az.from_pymc3(trace_1)
 
-        traces = [trace_0, trace_0]
-        models = [model_0, model_0]
-        ppc = pm.sample_posterior_predictive_w(traces, 100, models)
-        assert ppc["y"].shape == (100, 500)
+        with pm.Model() as model_2:
+            # Model with no observed RVs.
+            mu = pm.Normal("mu", mu=0, sigma=1)
+            with pytest.warns(UserWarning, match=warning_msg):
+                trace_2 = pm.sample(10, tune=0, return_inferencedata=False)
 
         traces = [trace_0, trace_1]
+        idatas = [idata_0, idata_1]
         models = [model_0, model_1]
+
         ppc = pm.sample_posterior_predictive_w(traces, 100, models)
-        assert ppc["y"].shape == (100, 500)
+        assert ppc["y"].shape == (100, 50)
+
+        ppc = pm.sample_posterior_predictive_w(idatas, 100, models)
+        assert ppc["y"].shape == (100, 50)
+
+        with model_0:
+            ppc = pm.sample_posterior_predictive_w([idata_0.posterior], None)
+            assert ppc["y"].shape == (20, 50)
+
+        with pytest.raises(ValueError, match="The number of traces and weights should be the same"):
+            pm.sample_posterior_predictive_w([idata_0.posterior], 100, models, weights=[0.5, 0.5])
+
+        with pytest.raises(ValueError, match="The number of models and weights should be the same"):
+            pm.sample_posterior_predictive_w([idata_0.posterior], 100, models)
+
+        with pytest.raises(
+            ValueError, match="The number of observed RVs should be the same for all models"
+        ):
+            pm.sample_posterior_predictive_w([trace_0, trace_2], 100, [model_0, model_2])
 
 
 @pytest.mark.parametrize(
@@ -766,16 +789,68 @@ def test_exec_nuts_init(method):
         assert "a" in start[0] and "b_log__" in start[0]
 
 
+@pytest.mark.parametrize(
+    "init, start, expectation",
+    [
+        ("auto", None, pytest.raises(SamplingError)),
+        ("jitter+adapt_diag", None, pytest.raises(SamplingError)),
+        ("auto", {"x": 0}, does_not_raise()),
+        ("jitter+adapt_diag", {"x": 0}, does_not_raise()),
+        ("adapt_diag", None, does_not_raise()),
+    ],
+)
+def test_default_sample_nuts_jitter(init, start, expectation, monkeypatch):
+    # This test tries to check whether the starting points returned by init_nuts are actually
+    # being used when pm.sample() is called without specifying an explicit start point (see
+    # https://github.com/pymc-devs/pymc3/pull/4285).
+    def _mocked_init_nuts(*args, **kwargs):
+        if init == "adapt_diag":
+            start_ = [{"x": np.array(0.79788456)}]
+        else:
+            start_ = [{"x": np.array(-0.04949886)}]
+        _, step = pm.init_nuts(*args, **kwargs)
+        return start_, step
+
+    monkeypatch.setattr("pymc3.sampling.init_nuts", _mocked_init_nuts)
+    with pm.Model() as m:
+        x = pm.HalfNormal("x", transform=None)
+        with expectation:
+            pm.sample(tune=1, draws=0, chains=1, init=init, start=start)
+
+
+@pytest.mark.parametrize(
+    "testval, jitter_max_retries, expectation",
+    [
+        (0, 0, pytest.raises(SamplingError)),
+        (0, 1, pytest.raises(SamplingError)),
+        (0, 4, does_not_raise()),
+        (0, 10, does_not_raise()),
+        (1, 0, does_not_raise()),
+    ],
+)
+def test_init_jitter(testval, jitter_max_retries, expectation):
+    with pm.Model() as m:
+        pm.HalfNormal("x", transform=None, testval=testval)
+
+    with expectation:
+        # Starting value is negative (invalid) when np.random.rand returns 0 (jitter = -1)
+        # and positive (valid) when it returns 1 (jitter = 1)
+        with mock.patch("numpy.random.rand", side_effect=[0, 0, 0, 1, 0]):
+            start = pm.sampling._init_jitter(m, chains=1, jitter_max_retries=jitter_max_retries)
+            pm.util.check_start_vals(start, m)
+
+
 @pytest.fixture(scope="class")
 def point_list_arg_bug_fixture() -> Tuple[pm.Model, pm.backends.base.MultiTrace]:
     with pm.Model() as pmodel:
-        n = pm.Normal('n')
+        n = pm.Normal("n")
         trace = pm.sample()
 
     with pmodel:
-        d = pm.Deterministic('d', n * 4)
+        d = pm.Deterministic("d", n * 4)
     return pmodel, trace
 
+
 class TestSamplePriorPredictive(SeededTest):
     def test_ignores_observed(self):
         observed = np.random.normal(10, 1, size=200)
@@ -793,9 +868,7 @@ class TestSamplePriorPredictive(SeededTest):
         assert (prior["positive_mu"] > 90).all()
         assert (prior["x_obs"] < 90).all()
         assert prior["x_obs"].shape == (500, 200)
-        npt.assert_array_almost_equal(
-            prior["positive_mu"], np.abs(prior["mu"]), decimal=4
-        )
+        npt.assert_array_almost_equal(prior["positive_mu"], np.abs(prior["mu"]), decimal=4)
 
     def test_respects_shape(self):
         for shape in (2, (2,), (10, 2), (10, 10)):
@@ -825,20 +898,14 @@ class TestSamplePriorPredictive(SeededTest):
             probs = pm.Dirichlet("probs", a=np.ones(6), shape=6)
             obs = pm.Multinomial("obs", n=100, p=probs, observed=mn_data)
             burned_trace = pm.sample(20, tune=10, cores=1)
-        sim_priors = pm.sample_prior_predictive(samples=20,
-                                                model=dm_model)
-        sim_ppc = pm.sample_posterior_predictive(burned_trace,
-                                                 samples=20,
-                                                 model=dm_model)
-        assert sim_priors['probs'].shape == (20, 6)
-        assert sim_priors['obs'].shape == (20,) + obs.distribution.shape
-        assert sim_ppc['obs'].shape == (20,) + obs.distribution.shape
-
-        sim_ppc = pm.fast_sample_posterior_predictive(burned_trace,
-                                                 samples=20,
-                                                 model=dm_model)
-        assert sim_ppc['obs'].shape == (20,) + obs.distribution.shape
+        sim_priors = pm.sample_prior_predictive(samples=20, model=dm_model)
+        sim_ppc = pm.sample_posterior_predictive(burned_trace, samples=20, model=dm_model)
+        assert sim_priors["probs"].shape == (20, 6)
+        assert sim_priors["obs"].shape == (20,) + obs.distribution.shape
+        assert sim_ppc["obs"].shape == (20,) + obs.distribution.shape
 
+        sim_ppc = pm.fast_sample_posterior_predictive(burned_trace, samples=20, model=dm_model)
+        assert sim_ppc["obs"].shape == (20,) + obs.distribution.shape
 
     def test_layers(self):
         with pm.Model() as model:
@@ -860,9 +927,7 @@ class TestSamplePriorPredictive(SeededTest):
             kappa_log = pm.Exponential("logkappa", lam=5.0)
             kappa = pm.Deterministic("kappa", tt.exp(kappa_log))
 
-            thetas = pm.Beta(
-                "thetas", alpha=phi * kappa, beta=(1.0 - phi) * kappa, shape=n
-            )
+            thetas = pm.Beta("thetas", alpha=phi * kappa, beta=(1.0 - phi) * kappa, shape=n)
 
             y = pm.Binomial("y", n=at_bats, p=thetas, observed=hits)
             gen = pm.sample_prior_predictive(draws)
@@ -891,7 +956,6 @@ class TestSamplePriorPredictive(SeededTest):
         assert gen2["y"].shape == (draws, n2)
 
     def test_density_dist(self):
-
         obs = np.random.normal(-1, 0.1, size=10)
         with pm.Model():
             mu = pm.Normal("mu", 0, 1)
@@ -927,30 +991,23 @@ class TestSamplePriorPredictive(SeededTest):
     def test_bounded_dist(self):
         with pm.Model() as model:
             BoundedNormal = pm.Bound(pm.Normal, lower=0.0)
-            x = BoundedNormal(
-                "x", mu=tt.zeros((3, 1)), sd=1 * tt.ones((3, 1)), shape=(3, 1)
-            )
+            x = BoundedNormal("x", mu=tt.zeros((3, 1)), sd=1 * tt.ones((3, 1)), shape=(3, 1))
 
         with model:
             prior_trace = pm.sample_prior_predictive(5)
             assert prior_trace["x"].shape == (5, 3, 1)
 
+
 class TestSamplePosteriorPredictive:
     def test_point_list_arg_bug_fspp(self, point_list_arg_bug_fixture):
         pmodel, trace = point_list_arg_bug_fixture
         with pmodel:
-            pp = pm.fast_sample_posterior_predictive(
-                [trace[15]],
-                var_names=['d']
-            )
+            pp = pm.fast_sample_posterior_predictive([trace[15]], var_names=["d"])
 
     def test_point_list_arg_bug_spp(self, point_list_arg_bug_fixture):
         pmodel, trace = point_list_arg_bug_fixture
         with pmodel:
-            pp = pm.sample_posterior_predictive(
-                [trace[15]],
-                var_names=['d']
-            )
+            pp = pm.sample_posterior_predictive([trace[15]], var_names=["d"])
 
     def test_sample_from_xarray_prior(self, point_list_arg_bug_fixture):
         pmodel, trace = point_list_arg_bug_fixture
@@ -959,25 +1016,16 @@ class TestSamplePosteriorPredictive:
             prior = pm.sample_prior_predictive(samples=20)
         idat = az.from_pymc3(trace, prior=prior)
         with pmodel:
-            pp = pm.sample_posterior_predictive(
-                idat.prior,
-                var_names=['d']
-            )
+            pp = pm.sample_posterior_predictive(idat.prior, var_names=["d"])
 
     def test_sample_from_xarray_posterior(self, point_list_arg_bug_fixture):
         pmodel, trace = point_list_arg_bug_fixture
         idat = az.from_pymc3(trace)
         with pmodel:
-            pp = pm.sample_posterior_predictive(
-                idat.posterior,
-                var_names=['d']
-            )
+            pp = pm.sample_posterior_predictive(idat.posterior, var_names=["d"])
 
     def test_sample_from_xarray_posterior_fast(self, point_list_arg_bug_fixture):
         pmodel, trace = point_list_arg_bug_fixture
         idat = az.from_pymc3(trace)
         with pmodel:
-            pp = pm.fast_sample_posterior_predictive(
-                idat.posterior,
-                var_names=['d']
-            )
+            pp = pm.fast_sample_posterior_predictive(idat.posterior, var_names=["d"])
